

// PatternViewerApplet.java
// Version 1.2 (9/27/97)
//
// Jack Boyce (jboyce@physics.berkeley.edu)
//
// See appnotes.html for release notes.

import java.applet.*;
import java.awt.*;
import java.util.*;
import java.io.*;
import java.net.*;



public class PatternViewerApplet extends Applet {
	protected PatternViewer pv;
//	protected JuggleAnimApplet animapplet;
	protected InputStream patternfile;
	
	public void init() {
		String patfilename = getParameter("patternfile");
		
		if (patfilename != null) {
			try {
				URL patURL = new URL(getDocumentBase(), patfilename);
				patternfile = patURL.openStream();
			}
			catch (MalformedURLException mue) {
				showStatus("Syntax error in pattern file URL '"+patfilename+"'");
			}
			catch (IOException ioe) {
				showStatus("Couldn't open pattern file");
			}
		}
		
//		String animname = getParameter("animtarget");
//		if (animname != null) {
//			try {
//				Object obj = this.getAppletContext().getApplet(animname);
//				if (obj == null)
//					System.out.println("can't find animator");
//				else {
//					System.out.println(obj.getClass().getName());
//					animapplet = (JuggleAnimApplet)obj;
//				}
//			}
//			catch (ClassCastException cce) {
//				System.out.println("Error: "+cce.getMessage());
//				animapplet = null;
//			}
//		}
		
		try {
//			if (animapplet == null)
				pv = new PatternViewer(patternfile, getParameter("input"));
//			else
//				pv = new PatternViewer(animapplet, patternfile, getParameter("input"));
			
			this.setLayout(new GridLayout(1,1));
			this.add(pv);
			this.layout();
		}
		catch (JuggleException je) {
			showStatus(je.getMessage());
		}
	}
}


class PatternViewerWindow extends Frame {
	protected PatternViewer pv;
	
	public PatternViewerWindow(String name, InputStream patternfile,
						String input) throws JuggleException {
		this(name, 500, 350, patternfile, input);
	}
						
	public PatternViewerWindow(String name, int width, int height, InputStream patternfile,
						String input) throws JuggleException {
		super(name);
		this.resize(width, height);
		this.setResizable(true);
		
		try {
			pv = new PatternViewer(patternfile, input);
			GridBagLayout gb = new GridBagLayout();
		
			this.setLayout(gb);
			this.add(pv);
			
			GridBagConstraints gbc1 = new GridBagConstraints();
			gbc1.anchor = GridBagConstraints.CENTER;
			gbc1.fill = GridBagConstraints.BOTH;
			gbc1.gridheight = gbc1.gridwidth = 1;
			gbc1.gridx = gbc1.gridy = 0;
			gbc1.insets = new Insets(10,10,10,10);
			gbc1.weightx = gbc1.weighty = 1.0;
			
			gb.setConstraints(pv, gbc1);
			this.layout();
			this.show();
		}
		catch (JuggleException je) {
			this.dispose();
			throw je;
		}
	}
	
	public boolean handleEvent(Event event) {
		if (event.id == Event.WINDOW_DESTROY) {
			this.dispose();
			return true;
		}
		else return super.handleEvent(event);
	}
	
	public static void main(String[] args) {
		String input = "animator=panel;generator=siteswap;editor=true";
		
		if (args.length == 0)
			System.out.println(
"Pattern Viewer, by Jack Boyce (9/27/97)\n\n" +
"These three optional command line arguments can be used:\n" +
"   <Applet 'input' parameter (no spaces)> <width> <height>\n"
			);
		
		if (args.length > 0)
			input = args[0];
		
		try {
			if (args.length > 2)
				new PatternViewerWindow("Pattern Viewer", Integer.parseInt(args[1]),
								Integer.parseInt(args[2]), null, input);
			else
				new PatternViewerWindow("Pattern VIewer", null, input);
		}
		catch (JuggleException je) {
			System.out.println(je.getMessage());
		}
		catch (NumberFormatException nfe) {
			System.out.println("Syntax error in window dimensions");
		}
	}
}


class PatternViewer extends Panel {
	protected final static int ANIMATOR_NONE = 0;
	protected final static int ANIMATOR_PANEL = 1;
	protected final static int ANIMATOR_WINDOW = 2;
	protected final static int ANIMATOR_MULTIPLEWINDOWS = 3;
//	protected final static int ANIMATOR_APPLET = 4;
	
	protected int animator;				// type of animator output
	protected boolean generator, lister, editor;
	protected boolean fileviewmode;
	protected String[] jugglercommands;	// for fileview mode
	protected Button control_button, go_button;   	//, animate_button;
	protected Label result_label;
	protected List pattern_list;
	protected TextField editor_text;
	protected Generator gen;	
	protected JuggleAnimWindow jaw;		// pop up window
//	protected JuggleAnimApplet jaa;		// tell applet to animate
	protected JuggleAnim ja;			// animate in a panel
	
		// constructor when animation commands are going to an applet
//	public PatternViewer(JuggleAnimApplet jaa, InputStream patternfile, String input)
//					throws JuggleException {
//		this(patternfile, input);
//		this.jaa = jaa;
//	}
				
	public PatternViewer(InputStream patternfile, String input) throws JuggleException {
		String generatorname = null;
		
		fileviewmode = (patternfile != null) ? true : false;
			
		generator = false;			// defaults
		editor = false;
		lister = true;
		animator = fileviewmode ? ANIMATOR_PANEL : ANIMATOR_NONE;
		
			// parse the input string
		if (input != null) {
			StringTokenizer st3 = new StringTokenizer(input, ";");
			int numparams = st3.countTokens();
			String param, value;
			
			for (int index = 0; index < numparams; index++) {
				StringTokenizer st4 = new StringTokenizer(st3.nextToken(), "=");
				if (st4.countTokens() != 2)
					throw new JuggleException("Syntax error in input");
					
				param = st4.nextToken().trim();
				value = st4.nextToken().trim();
				
				if (param.equalsIgnoreCase("generator")) {
					if (!fileviewmode) {
						generatorname = value;
						generator = true;
					}
				} else if (param.equalsIgnoreCase("editor")) {
					if (!fileviewmode) {
						if (value.equalsIgnoreCase("true"))
							editor = true;
						else if (value.equalsIgnoreCase("false"))
							editor = false;
						else
							throw new JuggleException("Unrecognized editor setting");
					}
				} else if (param.equalsIgnoreCase("animator")) {
					if (value.equalsIgnoreCase("none"))
						animator = ANIMATOR_NONE;
					else if (value.equalsIgnoreCase("panel"))
						animator = ANIMATOR_PANEL;
					else if (value.equalsIgnoreCase("window"))
						animator = ANIMATOR_WINDOW;
					else if (value.equalsIgnoreCase("multiplewindows"))
						animator = ANIMATOR_MULTIPLEWINDOWS;
					else
						throw new JuggleException("Unrecognized animator type");
				} else {
					throw new JuggleException("Unrecognized option in input");
				}
			}			
		}
		
			// Certain combinations don't make sense.  Enforce these.
		if (fileviewmode) {
			if (animator == ANIMATOR_NONE)
				animator = ANIMATOR_PANEL;
			generator = false;
			editor = false;
		}
		
		if (generator == false && fileviewmode == false)
			lister = false;			// nothing to list in this case
		
		if (animator == ANIMATOR_NONE)
			editor = false;
			
			// Get the generator object, if necessary
		if (generator)
			gen = Generator.getGenerator(generatorname);
		
			// Now that we know what GUI elements to make, go about making them
			
		GridBagLayout gb = new GridBagLayout();
		Font courierfont = new Font("Courier", Font.PLAIN, 10);
		Panel generatorpanel = null, editorpanel = null;
		
		if (generator) {
			generatorpanel = new Panel();
			generatorpanel.setLayout(gb);
			
			Panel temppanel = new Panel();
			temppanel.setLayout(new FlowLayout(FlowLayout.CENTER, 10, 10));
			control_button = new Button("Show Controls");
			temppanel.add(control_button);
			go_button = new Button("Generate!");
			temppanel.add(go_button);
			
			result_label = new Label(gen.getStartupMessage(), Label.LEFT);
			
			generatorpanel.add(temppanel);
			gb.setConstraints(temppanel, make_constraints(GridBagConstraints.CENTER,
						GridBagConstraints.NONE,0,0, new Insets(0,0,0,0)));
			generatorpanel.add(result_label);
			GridBagConstraints gbc4 = make_constraints(GridBagConstraints.WEST,
						GridBagConstraints.HORIZONTAL,0,1, new Insets(0,10,10,10));
			gbc4.weightx = 1.0;
			gb.setConstraints(result_label, gbc4);
		}

		if (lister) {
			pattern_list = new List();
			pattern_list.setMultipleSelections(false);
		
			if (fileviewmode)
				readfile(patternfile);		// read the file, if one is specified.
			else
				pattern_list.setFont(courierfont);
		}
		
		if (editor) {
			editorpanel = new Panel();
			editorpanel.setLayout(gb);
			
//			Label editor_label = new Label("Pattern:", Label.LEFT);
//			editorpanel.add(editor_label);
//			gb.setConstraints(editor_label, make_constraints(GridBagConstraints.WEST,
//						GridBagConstraints.NONE,0,0, new Insets(0,0,0,0)));
			
			editor_text = new TextField(20);
			editor_text.setFont(courierfont);
			editorpanel.add(editor_text);
			GridBagConstraints gbc3 = make_constraints(GridBagConstraints.WEST,
						GridBagConstraints.HORIZONTAL,1,0, new Insets(0,0,0,0));
			gbc3.weightx = 1.0;
			gb.setConstraints(editor_text, gbc3);
//			animate_button = new Button("Animate!");
//			editorpanel.add(animate_button);
//			gb.setConstraints(animate_button, make_constraints(GridBagConstraints.EAST,
//						GridBagConstraints.NONE,2,0, new Insets(0,0,0,0)));
		}
		
			// Now from these pieces construct the whole panel.  There are two
			// basic styles, depending on whether there's an animation panel or not.
			
		if (animator == ANIMATOR_PANEL) {
			ja = new JuggleAnim();		// JuggleAnim is a type of panel
			
			int gridwidth = (generator || lister) ? 2 : 1;
			this.setLayout(new GridLayout(1, gridwidth));
			
			Panel leftpanel;
			
			if (editor) {
				leftpanel = new Panel();
				leftpanel.setLayout(gb);
				leftpanel.add(ja);
				GridBagConstraints gbc1 = make_constraints(GridBagConstraints.CENTER,
							GridBagConstraints.BOTH,0,0, new Insets(0,0,0,10));
				gbc1.weightx = gbc1.weighty = 1.0;
				gb.setConstraints(ja, gbc1);
				leftpanel.add(editorpanel);
				GridBagConstraints gbc2 = make_constraints(GridBagConstraints.CENTER,
							GridBagConstraints.HORIZONTAL,0,1, new Insets(10,0,0,10));
				gbc2.weightx = 1.0;
				gb.setConstraints(editorpanel, gbc2);
			} else
				leftpanel = ja;
			
			this.add(leftpanel);
			
			if (gridwidth == 2) {
				Component rightcomp;
				
				if (generator) {
					Panel rightpanel = new Panel();
					rightpanel.setLayout(gb);
					rightpanel.add(generatorpanel);
					GridBagConstraints gbc1 = make_constraints(GridBagConstraints.WEST,
								GridBagConstraints.HORIZONTAL,0,0, new Insets(0,0,0,0));
					gbc1.weightx = 1.0;
					gb.setConstraints(generatorpanel, gbc1);
					
					rightpanel.add(pattern_list);
					GridBagConstraints gbc2 = make_constraints(GridBagConstraints.CENTER,
								GridBagConstraints.BOTH,0,1, new Insets(0,0,0,0));
					gbc2.weightx = gbc2.weighty = 1.0;
					gb.setConstraints(pattern_list, gbc2);
					rightcomp = (Component)rightpanel;
				} else
					rightcomp = pattern_list;
					
				this.add(rightcomp);
			}
		} else {		// otherwise, animations pop up in a window(s)
			this.setLayout(gb);
			if (generator) {
				this.add(generatorpanel);
				GridBagConstraints gbc1 = make_constraints(GridBagConstraints.WEST,
							GridBagConstraints.HORIZONTAL,0,0, new Insets(0,0,0,0));
				gbc1.weightx = 1.0;
				gb.setConstraints(generatorpanel, gbc1);
			}
			
			if (lister) {
				this.add(pattern_list);
				GridBagConstraints gbc2 = make_constraints(GridBagConstraints.CENTER,
							GridBagConstraints.BOTH,0,1, new Insets(0,0,0,0));
				gbc2.weightx = gbc2.weighty = 1.0;
				gb.setConstraints(pattern_list, gbc2);
			}
			
			if (editor) {
				this.add(editorpanel);
				GridBagConstraints gbc2 = make_constraints(GridBagConstraints.CENTER,
							GridBagConstraints.HORIZONTAL,0,2, new Insets(10,0,0,0));
				gbc2.weightx = 1.0;
				gb.setConstraints(editorpanel, gbc2);
			}
		}
	}
	
	protected static GridBagConstraints make_constraints(int location, int fill,
					int gridx, int gridy, Insets ins) {
		GridBagConstraints gbc = new GridBagConstraints();
		
		gbc.anchor = location;
		gbc.fill = fill;
		gbc.gridheight = gbc.gridwidth = 1;
		gbc.gridx = gridx;
		gbc.gridy = gridy;
		gbc.insets = ins;
		gbc.weightx = gbc.weighty = 0.0;
		return gbc;
	}
	
		// might want to make this public at some point
	protected void readfile(InputStream instream) {
		DataInputStream dis = new DataInputStream(instream);
			// erase the list and start reading file
		pattern_list.clear();
		jugglercommands = null;
		Vector v1 = new Vector(50,10);
		try {
			String currentline;
			StringTokenizer st;
			int numtokens;
			
			for (;;) {
				currentline = dis.readLine();
				if (currentline == null)
					break;
				st = new StringTokenizer(currentline, "\t");
				numtokens = st.countTokens();
				
				if (numtokens > 0)
					pattern_list.addItem(st.nextToken());
				else
					pattern_list.addItem(" ");
				
				if (numtokens > 1)
					v1.addElement(st.nextToken());
				else
					v1.addElement(null);
			}
		}
		catch (IOException ioe) {
		}
		try {
			dis.close();
		}
		catch (IOException ioe) {
		}
		int numlines = v1.size();
		jugglercommands = new String[numlines];
		Enumeration enum = v1.elements();
		for (int index = 0; index < numlines; index++)
			jugglercommands[index] = (String)enum.nextElement();
	}
	
	public boolean handleEvent(Event event) {
		if (event.id == Event.ACTION_EVENT) {
			if (generator && event.target == control_button) {
				gen.showControls();
				return true;
			} else if (generator && event.target == go_button) {
				try {
					gen.initGenerate();
					pattern_list.clear();
					int numpatterns = gen.runGenerate(pattern_list, 250);// limit of 250 patterns
					if (numpatterns == 0) {
						pattern_list.repaint(100);
						result_label.setText("No patterns found");
					}
					else if (numpatterns == 1)
						result_label.setText("1 pattern found");
					else
						result_label.setText(Integer.toString(numpatterns) + " patterns found");
				} catch (JuggleException je) {
					result_label.setText(je.getMessage());
				}
				return true;
			} else if (event.target == pattern_list) {
				if (fileviewmode == false && editor == true)
					animate_pattern((String)event.arg, null);
				return true;
			} else if (editor && (/*event.target == animate_button ||*/ event.target == editor_text)) {
				animate_pattern(editor_text.getText(), null);
				return true;
			}
		} else if (event.id == Event.LIST_SELECT) {
			int index = ((Integer)event.arg).intValue();

			if (editor)
				editor_text.setText(pattern_list.getItem(index).trim());
			else if (fileviewmode) {
				String temp = jugglercommands[index];
				if (temp != null)
					animate_pattern(pattern_list.getItem(index), temp);
			} else
				animate_pattern(pattern_list.getItem(index), null);
			
			return true;
		}
		
		return super.handleEvent(event);
	}
	
	protected void animate_pattern(String label, String pattern) {
		if (animator == ANIMATOR_NONE)
			return;
			
		String jugglerinput;
		
		if (fileviewmode)
			jugglerinput = pattern;
		else if (generator)
			jugglerinput = gen.virtual_makeAnimatorInput(label);
		else if (editor) {
			label = label.trim();
			
			if (label.indexOf((int)'=') == -1) {	// assume siteswap notation
				char firstchar = label.charAt(0);
				
				if (firstchar == '(')
					jugglerinput = "mode=syncsiteswap;pattern="+label+";slowdown=2.0";
				else if (firstchar == '<')
					return;		// can't do passing yet
				else
					jugglerinput = "mode=siteswap;pattern="+label+";slowdown=2.0";
			} else
				jugglerinput = label;		// user typing directly to animator
				
			label = "Pattern";
		} else
			return;			// shouldn't happen
		
		label = label.trim();
		
		switch (animator) {
			case ANIMATOR_PANEL:
				ja.restartJuggle(jugglerinput);		// output to panel
				break;
			case ANIMATOR_WINDOW:
				if (jaw == null)
					jaw = new JuggleAnimWindow(label, jugglerinput);
				else if (jaw.isAlive())
					jaw.restartJuggle(label, jugglerinput);
				else
					jaw = new JuggleAnimWindow(label, jugglerinput);
				break;
			case ANIMATOR_MULTIPLEWINDOWS:
				new JuggleAnimWindow(label, jugglerinput);
				break;
//			case ANIMATOR_APPLET:
//				jaa.restartJuggle(jugglerinput);
//				break;
		}
	}
	
	protected void finalize() {
		if (gen != null)
			gen.killControls();
	}
}

                         
//class JuggleException extends Exception {
//	JuggleException()  			{ super();	}
//	JuggleException(String s)	{ super(s);	}
//}

